@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: AddressTable.cpp,v 1.9 2006/05/17 02:27:33 vkurland Exp $
+  $Id: AddressTable.cpp,v 1.11 2006/09/10 22:40:36 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -51,7 +51,8 @@ AddressTable::AddressTable() : MultiAddress()
     setRunTime(false);
 }
 
-AddressTable::AddressTable(const FWObject *root) : MultiAddress(root) 
+AddressTable::AddressTable(const FWObject *root,bool prepopulate) :
+    MultiAddress(root,prepopulate) 
 {
     registerSourceAttributeName("filename");
     setSourceName("");
@@ -87,7 +88,7 @@ void AddressTable::loadFromSource() throw(FWException)
             {
                 try
                 {
-                    net=Network::cast(getRoot()->create(Network::TYPENAME,false));
+                    net=Network::cast(getRoot()->create(Network::TYPENAME));
                     *net=buf;
                     root->add(net);
                     net->setName(buf);
