@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: FWReference.cpp,v 1.4 2006/03/05 04:15:42 vkurland Exp $
+  $Id: FWReference.cpp,v 1.5 2006/09/10 22:40:37 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -58,7 +58,7 @@ FWReference::FWReference()
     setPointer(NULL);
 }
 
-FWReference::FWReference(const FWObject *root) : FWObject(root)
+FWReference::FWReference(const FWObject *root,bool prepopulate) : FWObject(root,prepopulate)
 {
     remStr("name");
     remStr("comment");
