@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: RuleElement.cpp,v 1.7 2006/03/06 04:00:02 vkurland Exp $
+  $Id: RuleElement.cpp,v 1.8 2006/09/10 22:40:37 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -60,7 +60,7 @@ RuleElement::RuleElement()
     setBool("neg",false);
 }
 
-RuleElement::RuleElement(const FWObject *root) : FWObject(root)
+RuleElement::RuleElement(const FWObject *root,bool prepopulate) : FWObject(root,prepopulate)
 {
     remStr("comment");
     remStr("name");
@@ -149,10 +149,19 @@ void RuleElement::reset()
 void RuleElement::_initialize(const FWObject *root)
 {
     setRoot(root);
+
+/* we need to add a reference to 'any' to this new rule
+   element. However, when objects are being loaded from XML file, ANY
+   element may be located at its bottom so at the moment when this
+   rule element is created, we may have not read the ANY yet. In that
+   case we do not add the reference here because it will be added when
+   it is read from the file anyway
+*/
+
     string any_id=getAnyElementId();
-    FWObject *any_obj=((FWObjectDatabase*)root)->findInIndex(any_id);
-    assert (any_obj!=NULL);
-    FWObject::addRef( any_obj );
+    FWObject *any_obj=((FWObjectDatabase*)root)->checkIndex(any_id);
+    if (any_obj)
+        FWObject::addRef( any_obj );
 }
 
 string RuleElement::getAnyElementId()
@@ -184,9 +193,11 @@ void RuleElement::toggleNeg()
 
 const char *RuleElementSrc::TYPENAME={"Src"};
 RuleElementSrc::RuleElementSrc() {}
-RuleElementSrc::RuleElementSrc(const FWObject *root) : ObjectGroup(root),RuleElement(root) 
+RuleElementSrc::RuleElementSrc(const FWObject *root,bool prepopulate) :
+    ObjectGroup(root,prepopulate),RuleElement(root,prepopulate) 
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementSrc::getAnyElementId() {
@@ -197,9 +208,11 @@ string RuleElementSrc::getAnyElementId() {
 
 const char *RuleElementDst::TYPENAME={"Dst"};
 RuleElementDst::RuleElementDst() {}
-RuleElementDst::RuleElementDst(const FWObject *root) : ObjectGroup(root),RuleElement(root)
+RuleElementDst::RuleElementDst(const FWObject *root,bool prepopulate) :
+    ObjectGroup(root,prepopulate),RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementDst::getAnyElementId() {
@@ -210,9 +223,11 @@ string RuleElementDst::getAnyElementId() {
 
 const char *RuleElementSrv::TYPENAME={"Srv"};
 RuleElementSrv::RuleElementSrv() {}
-RuleElementSrv::RuleElementSrv(const FWObject *root) : ServiceGroup(root),RuleElement(root)
+RuleElementSrv::RuleElementSrv(const FWObject *root,bool prepopulate) :
+    ServiceGroup(root,prepopulate),RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 
@@ -222,9 +237,11 @@ string RuleElementSrv::getAnyElementId() {
 
 const char *RuleElementItf::TYPENAME={"Itf"};
 RuleElementItf::RuleElementItf() {}
-RuleElementItf::RuleElementItf(const FWObject *root) : RuleElement(root)
+RuleElementItf::RuleElementItf(const FWObject *root,bool prepopulate) :
+    RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementItf::getAnyElementId()
@@ -284,9 +301,11 @@ bool RuleElementItf::checkItfChildOfThisFw(FWObject *o)
 
 const char *RuleElementOSrc::TYPENAME={"OSrc"};
 RuleElementOSrc::RuleElementOSrc() {}
-RuleElementOSrc::RuleElementOSrc(const FWObject *root) : RuleElement(root)
+RuleElementOSrc::RuleElementOSrc(const FWObject *root,bool prepopulate) :
+    RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementOSrc::getAnyElementId() {
@@ -296,9 +315,11 @@ string RuleElementOSrc::getAnyElementId() {
 
 const char *RuleElementODst::TYPENAME={"ODst"};
 RuleElementODst::RuleElementODst() {}
-RuleElementODst::RuleElementODst(const FWObject *root) : RuleElement(root)
+RuleElementODst::RuleElementODst(const FWObject *root,bool prepopulate) :
+    RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementODst::getAnyElementId() {
@@ -308,9 +329,11 @@ string RuleElementODst::getAnyElementId() {
 
 const char *RuleElementOSrv::TYPENAME={"OSrv"};
 RuleElementOSrv::RuleElementOSrv() {}
-RuleElementOSrv::RuleElementOSrv(const FWObject *root) : RuleElement(root)
+RuleElementOSrv::RuleElementOSrv(const FWObject *root,bool prepopulate) :
+    RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementOSrv::getAnyElementId() {
@@ -322,9 +345,11 @@ string RuleElementOSrv::getAnyElementId() {
 
 const char *RuleElementTSrc::TYPENAME={"TSrc"};
 RuleElementTSrc::RuleElementTSrc() {}
-RuleElementTSrc::RuleElementTSrc(const FWObject *root) : RuleElement(root)
+RuleElementTSrc::RuleElementTSrc(const FWObject *root,bool prepopulate) :
+    RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementTSrc::getAnyElementId() {
@@ -334,9 +359,11 @@ string RuleElementTSrc::getAnyElementId() {
 
 const char *RuleElementTDst::TYPENAME={"TDst"};
 RuleElementTDst::RuleElementTDst() {}
-RuleElementTDst::RuleElementTDst(const FWObject *root) : RuleElement(root)
+RuleElementTDst::RuleElementTDst(const FWObject *root,bool prepopulate) :
+    RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementTDst::getAnyElementId() {
@@ -346,9 +373,11 @@ string RuleElementTDst::getAnyElementId() {
 
 const char *RuleElementTSrv::TYPENAME={"TSrv"};
 RuleElementTSrv::RuleElementTSrv()  {}
-RuleElementTSrv::RuleElementTSrv(const FWObject *root) : RuleElement(root)
+RuleElementTSrv::RuleElementTSrv(const FWObject *root,bool prepopulate) :
+    RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementTSrv::getAnyElementId() {
@@ -360,9 +389,11 @@ string RuleElementTSrv::getAnyElementId() {
 
 const char *RuleElementInterval::TYPENAME={"When"};
 RuleElementInterval::RuleElementInterval() {}
-RuleElementInterval::RuleElementInterval(const FWObject *root) : RuleElement(root)
+RuleElementInterval::RuleElementInterval(const FWObject *root,bool prepopulate) :
+    RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementInterval::getAnyElementId() {
@@ -374,9 +405,11 @@ string RuleElementInterval::getAnyElementId() {
 
 const char *RuleElementRDst::TYPENAME={"RDst"};
 RuleElementRDst::RuleElementRDst() {}
-RuleElementRDst::RuleElementRDst(const FWObject *root) : RuleElement(root)
+RuleElementRDst::RuleElementRDst(const FWObject *root,bool prepopulate) :
+    RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementRDst::getAnyElementId() {
@@ -385,9 +418,11 @@ string RuleElementRDst::getAnyElementId() {
 
 const char *RuleElementRGtw::TYPENAME={"RGtw"};
 RuleElementRGtw::RuleElementRGtw() {}
-RuleElementRGtw::RuleElementRGtw(const FWObject *root) : RuleElement(root)
+RuleElementRGtw::RuleElementRGtw(const FWObject *root,bool prepopulate) :
+    RuleElement(root,prepopulate)
 {
-    _initialize(root);
+    if (prepopulate)
+        _initialize(root);
 }
 
 string RuleElementRGtw::getAnyElementId() {
@@ -516,7 +551,8 @@ bool RuleElementRGtw::checkReachableIPAdress(FWObject *o) {
 
 const char *RuleElementRItf::TYPENAME={"RItf"};
 RuleElementRItf::RuleElementRItf()  {}
-RuleElementRItf::RuleElementRItf(const FWObject *root) : RuleElementItf(root) {}
+RuleElementRItf::RuleElementRItf(const FWObject *root,bool prepopulate) :
+    RuleElementItf(root,prepopulate) {}
 
 /**
  * Unlike RuleElementItf, RuleElementRItf can hold only single object.
