@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Tools.cpp,v 1.5 2005/04/25 05:39:19 vkurland Exp $
+  $Id: Tools.cpp,v 1.7 2006/11/06 17:24:50 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -47,6 +47,7 @@ struct __dirstream {};
 #  include <basetsd.h>
 #  include <io.h>
 #  include <windows.h>
+#  include <winsock2.h>
 #endif
 
 
@@ -184,6 +185,31 @@ int  libfwbuilder::cxx_strcasecmp(const char *s1, const char *s2)
 
 void libfwbuilder::init()
 {
+#ifdef _WIN32
+    WORD wVersionRequested;
+    WSADATA wsaData;
+    int err;
+ 
+    wVersionRequested = MAKEWORD( 2, 0 );
+ 
+    err = WSAStartup( wVersionRequested, &wsaData );
+    if ( err != 0 )
+    {
+        // Could not find usable dll
+        cerr << "Could not initialize winsock dll v2.0";
+        return;
+    }
+ 
+    // Confirm that the WinSock DLL supports 2.0.
+ 
+    if ( LOBYTE( wsaData.wVersion ) != 2 ||
+         HIBYTE( wsaData.wVersion ) != 0 )
+    {
+        cerr << "Could not initialize winsock dll v2.0";
+        WSACleanup( );
+        return; 
+    }
+#endif
 
     /* Initialize classes which need static initialization */
     DNS::init();
