@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: dns.cpp,v 1.8 2006/06/25 04:31:26 vkurland Exp $
+  $Id: dns.cpp,v 1.11 2006/11/06 16:54:51 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -352,7 +352,15 @@ list<IPAddress> DNS::getHostByName(const string &name) throw(FWException)
     if(!hp)
     {
         gethostbyname_mutex->unlock();
-        throw FWException("Host or network '"+name+"' not found");
+
+        std::ostringstream strerr;
+        strerr << "Host or network '"+name+"' not found; last error: ";
+#ifndef _WIN32
+        strerr << strerror(errno);
+#else
+        strerr << WSAGetLastError();
+#endif
+        throw FWException(strerr.str());
     }
 
 #endif
