@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: OSConfigurator.cpp,v 1.2 2004/09/08 05:34:53 vkurland Exp $
+  $Id: OSConfigurator.cpp,v 1.3 2006/03/07 07:28:28 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -33,6 +33,9 @@ using namespace std;
 OSConfigurator::~OSConfigurator() {}
 
 OSConfigurator::OSConfigurator(FWObjectDatabase *_db, const string &fwname) :
-    Compiler(_db,fwname)  {};
+    Compiler(_db,fwname)
+{
+    num_virtual_addresses_for_nat = 0;
+};
 
 
