@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler.cpp,v 1.16 2006/09/10 22:40:37 vkurland Exp $
+  $Id: PolicyCompiler.cpp,v 1.17 2006/09/21 06:15:21 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -162,6 +162,16 @@ bool PolicyCompiler::checkForShadowing(const PolicyRule &r1,const PolicyRule &r2
     if (r1.getAction()==PolicyRule::Accounting  ||
         r2.getAction()==PolicyRule::Accounting ) return false;
 
+    /*
+     * this is delicate case: negation. We consider r2 to "shade" r1
+     * only if r2 is above r1 in the policy; if r1 originally had
+     * negation and has been split, such as for example done in
+     * fwb_ipt, then some of the produced rules have action Return. If
+     * r2 has action != Return and r1 has action Return, we ignore r1.
+     */
+    if (r1.getAction()==PolicyRule::Return  ||
+        r2.getAction()==PolicyRule::Return ) return false;
+
     /*
      * TODO: actually, route rule may shadow other rules if it
      * translates into "final" target, that is stops processing. This
